@@ -36,6 +36,7 @@ from binascii import unhexlify
 
 from Cryptodome.SelfTest.st_common import list_test_cases
 from Cryptodome.Util.py3compat import bord, tostr, FileNotFoundError
+from Cryptodome.Util.asn1 import DerSequence, DerBitString
 from Cryptodome.Util.number import bytes_to_long
 from Cryptodome.Hash import SHAKE128
 
@@ -83,6 +84,19 @@ def compact(lines):
     return unhexlify(tostr(ext).replace(" ", "").replace(":", ""))
 
 
+def create_ref_keys_p224():
+    key_len = 28
+    key_lines = load_file("ecc_p224.txt").splitlines()
+    private_key_d = bytes_to_long(compact(key_lines[2:4]))
+    public_key_xy = compact(key_lines[5:9])
+    assert bord(public_key_xy[0]) == 4  # Uncompressed
+    public_key_x = bytes_to_long(public_key_xy[1:key_len+1])
+    public_key_y = bytes_to_long(public_key_xy[key_len+1:])
+
+    return (ECC.construct(curve="P-224", d=private_key_d),
+            ECC.construct(curve="P-224", point_x=public_key_x, point_y=public_key_y))
+
+
 def create_ref_keys_p256():
     key_len = 32
     key_lines = load_file("ecc_p256.txt").splitlines()
@@ -95,6 +109,7 @@ def create_ref_keys_p256():
     return (ECC.construct(curve="P-256", d=private_key_d),
             ECC.construct(curve="P-256", point_x=public_key_x, point_y=public_key_y))
 
+
 def create_ref_keys_p384():
     key_len = 48
     key_lines = load_file("ecc_p384.txt").splitlines()
@@ -107,6 +122,7 @@ def create_ref_keys_p384():
     return (ECC.construct(curve="P-384", d=private_key_d),
             ECC.construct(curve="P-384", point_x=public_key_x, point_y=public_key_y))
 
+
 def create_ref_keys_p521():
     key_len = 66
     key_lines = load_file("ecc_p521.txt").splitlines()
@@ -126,12 +142,130 @@ def get_fixed_prng():
     return SHAKE128.new().update(b"SEED").read
 
 
+def extract_bitstring_from_spki(data):
+        seq = DerSequence()
+        seq.decode(data)
+        bs = DerBitString()
+        bs.decode(seq[1])
+        return bs.value
+
+
 class TestImport(unittest.TestCase):
 
     def test_empty(self):
         self.assertRaises(ValueError, ECC.import_key, b"")
 
 
+class TestImport_P224(unittest.TestCase):
+
+    def __init__(self, *args, **kwargs):
+        super(TestImport_P224, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p224()
+
+    def test_import_public_der(self):
+        key_file = load_file("ecc_p224_public.der")
+
+        key = ECC._import_subjectPublicKeyInfo(key_file)
+        self.assertEqual(self.ref_public, key)
+
+        key = ECC._import_der(key_file, None)
+        self.assertEqual(self.ref_public, key)
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p224_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P224')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p224_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P224')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_private_der(self):
+        key_file = load_file("ecc_p224_private.der")
+
+        key = ECC._import_private_der(key_file, None)
+        self.assertEqual(self.ref_private, key)
+
+        key = ECC._import_der(key_file, None)
+        self.assertEqual(self.ref_private, key)
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_private_pkcs8_clear(self):
+        key_file = load_file("ecc_p224_private_p8_clear.der")
+
+        key = ECC._import_der(key_file, None)
+        self.assertEqual(self.ref_private, key)
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_private_pkcs8_in_pem_clear(self):
+        key_file = load_file("ecc_p224_private_p8_clear.pem")
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_private_pkcs8_encrypted_1(self):
+        key_file = load_file("ecc_p224_private_p8.der")
+
+        key = ECC._import_der(key_file, "secret")
+        self.assertEqual(self.ref_private, key)
+
+        key = ECC.import_key(key_file, "secret")
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_private_pkcs8_encrypted_2(self):
+        key_file = load_file("ecc_p224_private_p8.pem")
+
+        key = ECC.import_key(key_file, "secret")
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_x509_der(self):
+        key_file = load_file("ecc_p224_x509.der")
+
+        key = ECC._import_der(key_file, None)
+        self.assertEqual(self.ref_public, key)
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_public_pem(self):
+        key_file = load_file("ecc_p224_public.pem")
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_private_pem(self):
+        key_file = load_file("ecc_p224_private.pem")
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_private, key)
+
+    def test_import_private_pem_encrypted(self):
+        for algo in "des3", "aes128", "aes192", "aes256", "aes256_gcm":
+            key_file = load_file("ecc_p224_private_enc_%s.pem" % algo)
+
+            key = ECC.import_key(key_file, "secret")
+            self.assertEqual(self.ref_private, key)
+
+            key = ECC.import_key(tostr(key_file), b"secret")
+            self.assertEqual(self.ref_private, key)
+
+    def test_import_x509_pem(self):
+        key_file = load_file("ecc_p224_x509.pem")
+
+        key = ECC.import_key(key_file)
+        self.assertEqual(self.ref_public, key)
+
+
 class TestImport_P256(unittest.TestCase):
 
     def __init__(self, *args, **kwargs):
@@ -150,6 +284,18 @@ class TestImport_P256(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P256')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p256_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P256')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p256_private.der")
 
@@ -278,6 +424,18 @@ class TestImport_P384(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P384')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p384_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P384')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p384_private.der")
 
@@ -401,6 +559,18 @@ class TestImport_P521(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P521')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p521_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P521')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p521_private.der")
 
@@ -532,6 +702,21 @@ class TestExport_P256(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p256_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p256_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p256_private.der")
 
@@ -662,15 +847,15 @@ class TestExport_P256(unittest.TestCase):
         key_file = load_file("ecc_p256_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p256_public_openssh.txt", "rt")
@@ -678,7 +863,7 @@ class TestExport_P256(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -690,7 +875,7 @@ class TestExport_P256(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -702,7 +887,7 @@ class TestExport_P256(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -713,7 +898,7 @@ class TestExport_P256(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -746,22 +931,6 @@ class TestExport_P256(unittest.TestCase):
         self.assertRaises(ValueError, self.ref_private.export_key, format="OpenSSH",
                                       passphrase="secret")
 
-    def test_unsupported_curve(self):
-
-        # openssl ecparam -name secp224r1 -genkey -noout -out strange-curve.pem -conv_form uncompressed
-        curve = """-----BEGIN EC PRIVATE KEY-----
-MGgCAQEEHEi7xTHW+5oT8wgpjoEKV7uwMuY8rt2YUZe4j1SgBwYFK4EEACGhPAM6
-AATJgfOG+Bnki8robpNM8MtArji43GU9up4B0x9sVhqB+fZP+hXgV9ITN7YX4E/k
-gVnJp9EBND/tHQ==
------END EC PRIVATE KEY-----"""
-
-        from Cryptodome.PublicKey.ECC import UnsupportedEccFeature
-        try:
-            ECC.import_key(curve)
-        except UnsupportedEccFeature as uef:
-            assert("1.3.132.0.33" in str(uef))
-        else:
-            assert(False)
 
     def test_compressed_curve(self):
 
@@ -812,6 +981,21 @@ class TestExport_P384(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p384_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p384_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p384_private.der")
 
@@ -942,15 +1126,15 @@ class TestExport_P384(unittest.TestCase):
         key_file = load_file("ecc_p384_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p384_public_openssh.txt", "rt")
@@ -958,7 +1142,7 @@ class TestExport_P384(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -970,7 +1154,7 @@ class TestExport_P384(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -982,7 +1166,7 @@ class TestExport_P384(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -993,7 +1177,7 @@ class TestExport_P384(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -1081,6 +1265,21 @@ class TestExport_P521(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p521_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p521_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p521_private.der")
 
@@ -1211,15 +1410,15 @@ class TestExport_P521(unittest.TestCase):
         key_file = load_file("ecc_p521_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p521_public_openssh.txt", "rt")
@@ -1227,7 +1426,7 @@ class TestExport_P521(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -1239,7 +1438,7 @@ class TestExport_P521(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -1251,7 +1450,7 @@ class TestExport_P521(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -1262,7 +1461,7 @@ class TestExport_P521(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -1330,6 +1529,7 @@ def get_tests(config={}):
     tests = []
     tests += list_test_cases(TestImport)
     try:
+        tests += list_test_cases(TestImport_P224)
         tests += list_test_cases(TestImport_P256)
         tests += list_test_cases(TestImport_P384)
         tests += list_test_cases(TestImport_P521)
